@@ -29,6 +29,8 @@
 #include "parser/parse_relation.h"
 
 #include "replication/origin.h"
+#include "replication/reorderbuffer.h"
+
 #include "storage/bufmgr.h"
 #include "storage/lmgr.h"
 
@@ -745,7 +747,7 @@ tuple_to_stringinfo(StringInfo s, TupleDesc tupdesc, HeapTuple tuple)
 		char	   *outputstr = NULL;
 		bool		isnull;		/* column is null? */
 
-		attr = tupdesc->attrs[natt];
+		attr = TupleDescAttr(tupdesc, natt);
 
 		/*
 		 * don't print dropped columns, we can't be sure everything is
