@@ -13,6 +13,7 @@
 
 #include "postgres.h"
 
+#include "access/commit_ts.h"
 #include "access/genam.h"
 #include "access/heapam.h"
 #include "access/htup_details.h"
@@ -66,6 +67,8 @@
 #include "utils/rel.h"
 #include "utils/snapmgr.h"
 
+#include "pgstat.h"
+
 #include "pglogical_dependency.h"
 #include "pglogical_node.h"
 #include "pglogical_executor.h"
@@ -102,6 +105,9 @@ PG_FUNCTION_INFO_V1(pglogical_alter_subscription_resynchronize_table);
 PG_FUNCTION_INFO_V1(pglogical_show_subscription_table);
 PG_FUNCTION_INFO_V1(pglogical_show_subscription_status);
 
+PG_FUNCTION_INFO_V1(pglogical_wait_for_subscription_sync_complete);
+PG_FUNCTION_INFO_V1(pglogical_wait_for_table_sync_complete);
+
 /* Replication set manipulation. */
 PG_FUNCTION_INFO_V1(pglogical_create_replication_set);
 PG_FUNCTION_INFO_V1(pglogical_alter_replication_set);
@@ -134,6 +140,8 @@ PG_FUNCTION_INFO_V1(pglogical_version_num);
 PG_FUNCTION_INFO_V1(pglogical_min_proto_version);
 PG_FUNCTION_INFO_V1(pglogical_max_proto_version);
 
+PG_FUNCTION_INFO_V1(pglogical_xact_commit_timestamp_origin);
+
 static void gen_slot_name(Name slot_name, char *dbname,
 						  const char *provider_name,
 						  const char *subscriber_name);
@@ -335,6 +343,8 @@ pglogical_alter_node_drop_interface(PG_FUNCTION_ARGS)
 	char	   *if_name = NameStr(*PG_GETARG_NAME(1));
 	PGLogicalNode	   *node;
 	PGlogicalInterface *oldif;
+	List		   *other_subs;
+	ListCell	   *lc;
 
 	node = get_node_by_name(node_name, false);
 	if (node == NULL)
@@ -349,6 +359,18 @@ pglogical_alter_node_drop_interface(PG_FUNCTION_ARGS)
 				 errmsg("interface \"%s\" for node node \"%s\" not found",
 				 if_name, node_name)));
 
+	other_subs = get_node_subscriptions(node->id, true);
+	foreach (lc, other_subs)
+	{
+		PGLogicalSubscription  *sub = (PGLogicalSubscription *) lfirst(lc);
+		if (oldif->id == sub->origin_if->id)
+			ereport(ERROR,
+					(errcode(ERRCODE_OBJECT_NOT_IN_PREREQUISITE_STATE),
+					 errmsg("cannot drop interface \"%s\" for node \"%s\" because subscription \"%s\" is using it",
+							oldif->name, node->name, sub->name),
+					 errhint("change the subscription interface first")));
+        }
+
 	drop_node_interface(oldif->id);
 
 	PG_RETURN_BOOL(true);
@@ -599,7 +621,7 @@ pglogical_drop_subscription(PG_FUNCTION_ARGS)
 		/* Drop the origin tracking locally. */
 		originid = replorigin_by_name(sub->slot_name, true);
 		if (originid != InvalidRepOriginId)
-			replorigin_drop(originid);
+			pgl_replorigin_drop(originid);
 	}
 
 	PG_RETURN_BOOL(sub != NULL);
@@ -845,7 +867,7 @@ pglogical_alter_subscription_synchronize(PG_FUNCTION_ARGS)
 }
 
 /*
- * Resyncrhonize one existing table.
+ * Resynchronize one existing table.
  */
 Datum
 pglogical_alter_subscription_resynchronize_table(PG_FUNCTION_ARGS)
@@ -1898,7 +1920,7 @@ pglogical_show_repset_table_info(PG_FUNCTION_ARGS)
 	/* Build the column list. */
 	for (i = 0; i < reldesc->natts; i++)
 	{
-		Form_pg_attribute att = reldesc->attrs[i];
+		Form_pg_attribute att = TupleDescAttr(reldesc,i);
 
 		/* Skip dropped columns. */
 		if (att->attisdropped)
@@ -2095,6 +2117,162 @@ pglogical_table_data_filtered(PG_FUNCTION_ARGS)
 	PG_RETURN_NULL();
 }
 
+
+
+/*
+ * Wait for subscription and initial sync to complete, or, if relation info is
+ * given, for sync to complete for a specific table.
+ *
+ * We have to play games with snapshots to achieve this, since we're looking at
+ * pglogical tables in the future as far as our snapshot is concerned.
+ */
+static void
+pglogical_wait_for_sync_complete(char *subscription_name, char *relnamespace, char *relname)
+{
+	PGLogicalSubscription *sub;
+
+	/*
+	 * If we wait in SERIALIZABLE, then the next snapshot after we return
+	 * won't reflect the new state.
+	 */
+	if (IsolationUsesXactSnapshot())
+		elog(ERROR, "cannot wait for sync in REPEATABLE READ or SERIALIZABLE isolation");
+
+	sub = get_subscription_by_name(subscription_name, false);
+
+	do
+	{
+		PGLogicalSyncStatus	   *subsync;
+		List				   *tables;
+		bool					isdone = false;
+
+		/* We need to see the latest rows */
+		PushActiveSnapshot(GetLatestSnapshot());
+
+		subsync = get_subscription_sync_status(sub->id, true);
+		isdone = subsync && subsync->status == SYNC_STATUS_READY;
+		free_sync_status(subsync);
+
+		if (isdone)
+		{
+			/*
+			 * Subscription its self is synced, but what about separately
+			 * synced tables?
+			 */
+			if (relname != NULL)
+			{
+				PGLogicalSyncStatus *table = get_table_sync_status(sub->id, relnamespace, relname, true);
+				isdone = table && table->status == SYNC_STATUS_READY;
+				free_sync_status(table);
+			}
+			else
+			{
+				/*
+				 * This is plenty inefficient and we should probably just do a direct catalog
+				 * scan, but meh, it hardly has to be fast.
+				 */
+				ListCell *lc;
+				tables = get_unsynced_tables(sub->id);
+				isdone = tables == NIL;
+				foreach (lc, tables)
+				{
+					PGLogicalSyncStatus *table = lfirst(lc);
+					free_sync_status(table);
+				}
+				list_free(tables);
+			}
+		}
+
+		PopActiveSnapshot();
+
+		if (isdone)
+			break;
+
+		CHECK_FOR_INTERRUPTS();
+
+		/* some kind of backoff could be useful here */
+		(void) WaitLatch(&MyProc->procLatch,
+						 WL_LATCH_SET | WL_TIMEOUT, 200L);
+
+		ResetLatch(&MyProc->procLatch);
+	} while (1);
+}
+
+Datum
+pglogical_wait_for_subscription_sync_complete(PG_FUNCTION_ARGS)
+{
+	char *subscription_name = NameStr(*PG_GETARG_NAME(0));
+
+	pglogical_wait_for_sync_complete(subscription_name, NULL, NULL);
+
+	PG_RETURN_VOID();
+}
+
+Datum
+pglogical_wait_for_table_sync_complete(PG_FUNCTION_ARGS)
+{
+	char *subscription_name = NameStr(*PG_GETARG_NAME(0));
+	Oid relid = PG_GETARG_OID(1);
+	char *relname, *relnamespace;
+
+	relname = get_rel_name(relid);
+	relnamespace = get_namespace_name(get_rel_namespace(relid));
+
+	pglogical_wait_for_sync_complete(subscription_name, relnamespace, relname);
+
+	PG_RETURN_VOID();
+}
+
+/*
+ * Like pg_xact_commit_timestamp but extended for replorigin
+ * too.
+ */
+Datum
+pglogical_xact_commit_timestamp_origin(PG_FUNCTION_ARGS)
+{
+#ifdef HAVE_REPLICATION_ORIGINS
+	TransactionId xid = PG_GETARG_UINT32(0);
+	TimestampTz ts;
+	RepOriginId	origin;
+	bool		found;
+#endif
+	TupleDesc	tupdesc;
+	Datum		values[2];
+	bool		nulls[2] = {false, false};
+	HeapTuple	tup;
+
+	/*
+	 * Construct a tuple descriptor for the result row. Must match the
+	 * function declaration.
+	 */
+	tupdesc = CreateTemplateTupleDesc(2, false);
+	TupleDescInitEntry(tupdesc, (AttrNumber) 1, "timestamp",
+					   TIMESTAMPTZOID, -1, 0);
+	TupleDescInitEntry(tupdesc, (AttrNumber) 2, "roident",
+					   OIDOID, -1, 0);
+	tupdesc = BlessTupleDesc(tupdesc);
+
+#ifdef HAVE_REPLICATION_ORIGINS
+	found = TransactionIdGetCommitTsData(xid, &ts, &origin);
+
+	if (found)
+	{
+		values[0] = TimestampTzGetDatum(ts);
+		values[1] = ObjectIdGetDatum(origin);
+	}
+	else
+#endif
+	{
+		values[0] = (Datum)0;
+		nulls[0] = true;
+		values[1] = (Datum)0;
+		nulls[1] = true;
+	}
+
+	tup = heap_form_tuple(tupdesc, values, nulls);
+	PG_RETURN_DATUM(HeapTupleGetDatum(tup));
+}
+
 Datum
 pglogical_gen_slot_name(PG_FUNCTION_ARGS)
 {
