@@ -26,6 +26,8 @@
 #include "utils/resowner.h"
 #include "utils/timestamp.h"
 
+#include "pgstat.h"
+
 #include "pglogical_node.h"
 #include "pglogical_worker.h"
 #include "pglogical.h"
@@ -98,6 +100,9 @@ manage_apply_workers(void)
 			else
 				prev = wlc;
 		}
+		/* If the subscriber does not have a registered worker. */
+		if (!wlc)
+			apply = NULL;
 
 		/* Skip if the worker was alrady registered. */
 		if (pglogical_worker_running(apply))
