@@ -17,13 +17,8 @@
 #include <unistd.h>
 #include <dirent.h>
 
-#include "pglogical_output_plugin.h"
-
 #include "mb/pg_wchar.h"
 #include "replication/logical.h"
-#ifdef HAVE_REPLICATION_ORIGINS
-#include "replication/origin.h"
-#endif
 
 #include "access/xact.h"
 #include "executor/executor.h"
@@ -32,7 +27,9 @@
 #include "utils/inval.h"
 #include "utils/memutils.h"
 #include "utils/rel.h"
+#include "replication/origin.h"
 
+#include "pglogical_output_plugin.h"
 #include "pglogical.h"
 #include "pglogical_output_config.h"
 #include "pglogical_executor.h"
@@ -41,6 +38,10 @@
 #include "pglogical_queue.h"
 #include "pglogical_repset.h"
 
+#ifdef HAVE_REPLICATION_ORIGINS
+#include "replication/origin.h"
+#endif
+
 extern void		_PG_output_plugin_init(OutputPluginCallbacks *cb);
 
 static void pg_decode_startup(LogicalDecodingContext * ctx,
@@ -170,9 +171,7 @@ pg_decode_startup(LogicalDecodingContext * ctx, OutputPluginOptions *opt,
 	/* Short lived memory context for individual messages */
 	data->context = AllocSetContextCreate(ctx->context,
 										  "pglogical output msg context",
-										  ALLOCSET_DEFAULT_MINSIZE,
-										  ALLOCSET_DEFAULT_INITSIZE,
-										  ALLOCSET_DEFAULT_MAXSIZE);
+										  ALLOCSET_DEFAULT_SIZES);
 	data->allow_internal_basetypes = false;
 	data->allow_binary_basetypes = false;
 
@@ -852,9 +851,7 @@ relmetacache_init(MemoryContext decoding_context)
 
 		RelMetaCacheContext = AllocSetContextCreate(TopMemoryContext,
 											  "pglogical output relmetacache",
-											  ALLOCSET_DEFAULT_MINSIZE,
-											  ALLOCSET_DEFAULT_INITSIZE,
-											  ALLOCSET_DEFAULT_MAXSIZE);
+											  ALLOCSET_DEFAULT_SIZES);
 
 		/* Make a new hash table for the cache */
 		hash_flags = HASH_ELEM | HASH_CONTEXT;
@@ -992,35 +989,32 @@ relmetacache_prune(void)
 static void
 pglReorderBufferCleanSerializedTXNs(const char *slotname)
 {
-	DIR		   *spill_dir = NULL;
+	DIR		   *spill_dir;
 	struct dirent *spill_de;
 	struct stat statbuf;
 	char		path[MAXPGPATH * 2 + 12];
 
 	sprintf(path, "pg_replslot/%s", slotname);
 
-	/* we're only handling directories here, skip if it's not our's */
+	/* we're only handling directories here, skip if it's not ours */
 	if (lstat(path, &statbuf) == 0 && !S_ISDIR(statbuf.st_mode))
 		return;
 
 	spill_dir = AllocateDir(path);
-	while ((spill_de = ReadDir(spill_dir, path)) != NULL)
+	while ((spill_de = ReadDirExtended(spill_dir, path, INFO)) != NULL)
 	{
-		if (strcmp(spill_de->d_name, ".") == 0 ||
-			strcmp(spill_de->d_name, "..") == 0)
-			continue;
-
 		/* only look at names that can be ours */
 		if (strncmp(spill_de->d_name, "xid", 3) == 0)
 		{
-			sprintf(path, "pg_replslot/%s/%s", slotname,
-					spill_de->d_name);
+			snprintf(path, sizeof(path),
+					 "pg_replslot/%s/%s", slotname,
+					 spill_de->d_name);
 
 			if (unlink(path) != 0)
-				ereport(PANIC,
+				ereport(ERROR,
 						(errcode_for_file_access(),
-						 errmsg("could not remove file \"%s\": %m",
-								path)));
+						 errmsg("could not remove file \"%s\" during removal of pg_replslot/%s/*.xid: %m",
+								path, slotname)));
 		}
 	}
 	FreeDir(spill_dir);
