@@ -61,22 +61,19 @@ SELECT * FROM pglogical.replication_set_remove_table('default', 'basic_dml');
 SELECT * FROM pglogical.replication_set_add_table('default', 'basic_dml', true, row_filter := $rf$id > 1 AND data IS DISTINCT FROM 'baz' AND data IS DISTINCT FROM 'bbb'$rf$);
 
 -- fail, the membership in repset depends on data column
+\set VERBOSITY terse
 ALTER TABLE basic_dml DROP COLUMN data;
+\set VERBOSITY default
 
 SELECT pglogical.wait_slot_confirm_lsn(NULL, NULL);
 
 \c :subscriber_dsn
 
 -- wait for the initial data to copy
-DO $$
-BEGIN
-    FOR i IN 1..300 LOOP
-        IF NOT EXISTS (SELECT 1 FROM pglogical.local_sync_status WHERE sync_status NOT IN ('y', 'r')) THEN
-            EXIT;
-        END IF;
-        PERFORM pg_sleep(0.1);
-    END LOOP;
-END;$$;
+BEGIN;
+SET LOCAL statement_timeout = '10s';
+SELECT pglogical.wait_for_subscription_sync_complete('test_subscription');
+COMMIT;
 
 SELECT id, other, data, "SomeThing" FROM basic_dml ORDER BY id;
 
